#
# spec file for package cryfs
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


# https://www.cryptopp.com/wiki/Link_Time_Optimization
# see also -DCRYPTOPP_DISABLE_ASM=ON below
#%%define _lto_cflags %%{nil}

%bcond_without  tests

Name:           cryfs
Version:        1.0.1
Release:        2.2
Summary:        Cryptographic filesystem for the cloud
License:        LGPL-3.0-only
URL:            https://www.cryfs.org/
Source0:        https://github.com/cryfs/cryfs/releases/download/%{version}/%{name}-%{version}.tar.xz
Source1:        https://github.com/cryfs/cryfs/releases/download/%{version}/%{name}-%{version}.tar.xz.asc
# 0x5D5EC7BC6F1443EC2AF7177A9E6C996C991D25E1
Source2:        %{name}.keyring
# PATCH-FIX-UPSTREAM fix-build-with-boost-1_88.patch -- based on PR 494
Patch0:         fix-build-with-boost-1_88.patch
# PATCH-FIX-UPSTREAM fix-feature-fuse3.patch -- based on branch feature/fuse3
Patch1:         fix-feature-fuse3.patch
BuildRequires:  cmake >= 3.25
BuildRequires:  gcc-c++
BuildRequires:  libboost_atomic-devel-impl >= 1.84.0
BuildRequires:  libboost_chrono-devel-impl >= 1.84.0
BuildRequires:  libboost_filesystem-devel-impl >= 1.84.0
BuildRequires:  libboost_program_options-devel-impl >= 1.84.0
BuildRequires:  libboost_system-devel-impl >= 1.84.0
BuildRequires:  libboost_thread-devel-impl >= 1.84.0
BuildRequires:  ninja
BuildRequires:  pkgconfig
BuildRequires:  python3-base
BuildRequires:  cmake(range-v3)
BuildRequires:  cmake(spdlog)
BuildRequires:  pkgconfig(fuse3) >= 3.9.0
BuildRequires:  pkgconfig(libcurl)
BuildRequires:  pkgconfig(libssl)
%if %{with tests}
BuildRequires:  pkgconfig(gmock)
BuildRequires:  pkgconfig(gtest)
%endif
# system cryptopp lib cannot currently be used.
# see also https://github.com/cryfs/cryfs/issues/369
Provides:       bundled(libcryptopp) = 8.9.0

%description
CryFS provides a FUSE-based mount that encrypts file contents, file
sizes, metadata and directory structure. It uses encrypted same-size
blocks to store both the files themselves and the blocks' relations
to one another. These blocks are stored as individual files in the
base directory, which can then be synchronized to remote storage
(using an external tool).

%prep
%autosetup -c -p1

# install man pages also with default build type
sed -e '/Release/d' -i doc/CMakeLists.txt

%build
%define __builder ninja
%cmake \
%ifarch %{arm32} %{ix86} ppc64 ppc64le s390x
	-DCRYPTOPP_DISABLE_ASM=ON \
%endif
%if %{with tests}
	-DBUILD_TESTING=ON \
%endif
	-DBUILD_SHARED_LIBS=OFF \
	-DCMAKE_POSITION_INDEPENDENT_CODE=ON \
	-DCRYFS_UPDATE_CHECKS=OFF
%cmake_build

%install
%cmake_install

%check
export PATH=%{buildroot}%{_bindir}:$PATH
cryfs --version
%if %{with tests}
pushd build/test
./blobstore/blobstore-test
./blockstore/blockstore-test
#./cpp-utils/cpp-utils-test
%ifnarch %{ix86} ppc64 ppc64le
./cryfs/cryfs-test --gtest_filter='-CryConfigCompatibilityTest.*'
%endif
%ifnarch riscv64
./cryfs-cli/cryfs-cli-test --gtest_filter='-CliTest.*:CliTest_Unmount.*'
%endif
#./fspp/fspp-test
./gitversion/gitversion-test
./parallelaccessstore/parallelaccessstore-test
popd
%endif

%files
%license LICENSE.txt
%doc README.md ChangeLog.txt
%{_bindir}/%{name}
%{_bindir}/%{name}-unmount
%{_mandir}/man1/%{name}.1%{?ext_man}
%{_mandir}/man1/%{name}-unmount.1%{?ext_man}

%changelog
* Thu May  1 2025 munix9@googlemail.com
- Migration to fuse 3 and deprecation of fuse (1)
  * Add fix-feature-fuse3.patch -- based on upstream branch
    https://github.com/cryfs/cryfs/tree/feature/fuse3
    (bsc#1242083) (bsc#1242079)
- Increase physicalmemory for ppc64 and ppc64le builds
* Fri Apr 25 2025 munix9@googlemail.com
- Add _constraints (required for ppc64, ppc64le)
* Fri Apr 25 2025 munix9@googlemail.com
- Add fix-build-with-boost-1_88.patch
- Remove _constraints file (not needed for now)
- Cleanup spec file (from the previous 0.x versions):
  * Use __builder ninja
  * Use %%cmake macro
  * Add %%check/tests (arch specific)
  * Add -DCRYPTOPP_DISABLE_ASM=ON (arch specific)
* Sat Mar 22 2025 munix9@googlemail.com
- Update to version 1.0.1:
  Backwards Compatibility
  * Filesystems created with CryFS 0.11.x and CryFS 1.0.0 are fully
    compatible with each other. The storage format hasn't changed.
  * This means filesystems created with 0.10.x or 0.11.x can be
    mounted without requiring a migration.
  * Filesystems created with 1.0.0 or 0.11.x can be mounted by
    CryFS 0.10.x, but only if you configure it to use a cipher
    supported by CryFS 0.10.x, e.g. AES-256-GCM. The new default,
    XChaCha20-Poly1305, is not supported by CryFS 0.10.x.
  Fixes/Improvements
  * Added a man page for cryfs-unmount
  * Fixed small inaccuracy in calculation of free space in statvfs
  * Fix an issue when using -o atime mount options
- Remove 38849c22aa34c5fad10091e066a520dd831462b3.patch
  (merged upstream)
* Mon Nov  6 2023 Dominique Leuenberger <dimstar@opensuse.org>
-  Fix build with RPM 4.19: unnumbered patches are no longer
  supported.
* Sun Apr  2 2023 Bjørn Lie <bjorn.lie@gmail.com>
- Add upstream patch - Include stdexcept when using logic_error:
  * 38849c22aa34c5fad10091e066a520dd831462b3.patch
* Fri Oct 14 2022 Fabian Vogt <fvogt@suse.com>
- Update to v0.11.3:
  * Fixed build issue on systems with libfmt 9.0
  * Fixed build issue on Apple Silicon Macs
  * Fixed build issue on systems that only have python3 but no
    python executable
- Use python3-base instead of python for building
- Use %%autosetup
* Mon Aug  1 2022 Andreas Stieger <andreas.stieger@gmx.de>
- add upstream signing key and verify source signature
* Tue Feb 22 2022 Paolo Stivanin <info@paolostivanin.com>
- Update to v0.11.2:
  * Time to mount a file system was very long because the build
    didn't correctly use OpenMP. This is now fixed and file systems
    should open faster again.
  * Fix building of the range-v3 dependency. The conan remote URL
    for this dependency changed and we have to use the new URL.
  * Update to CryptoPP 8.6. This fixes a rare bug where
    CryptoPP 8.5 encrypts data wrongly.
  * cryfs-unmount correctly unmounts paths that contain spaces.
  * Updated to DokanY 1.2.2.1001.
* Sun Oct  3 2021 Andreas Stieger <andreas.stieger@gmx.de>
- remove cryptopp build dependency, library is bundled
* Mon Sep 27 2021 munix9@googlemail.com
- Update to upstream version 0.11.0
  * Backwards Compatibility:
  - Filesystems created with CryFS 0.10.x can be mounted without
    requiring a migration.
  - Filesystems created with CryFS 0.11.x can be mounted by CryFS
    0.10.x if you configure it to use a cipher supported by CryFS
    0.10.x, e.g. AES-256-GCM. The new default, XChaCha20-Poly1305,
    is not supported by CryFS 0.10.x.
  * Security:
  - Added the XChaCha20-Poly1305 encryption cipher. For new
    filesystems, this will be the default, but you're still able
    to create a filesystem with the previous default of AES-256-GCM
    by saying "no" to the "use default settings?" question when
    creating the file system. Also, old filesystems will not be
    automatically converted and will keep using AES-256-GCM.
    XChaCha20-Poly1305 is significantly slower than AES-256-GCM
    on modern CPUs, but it is more secure for large filesystems
    (>64GB).
    For AES-256-GCM, it is recommended to encrypt at most 2^32
    blocks, which at the CryFS default block size of 16KB would
    be 64GB. The more the filesystem grows above that, the more
    likely it gets that a nonce gets reused and the two
    corresponding blocks become decryptable by an adversary.
    Other blocks would not be affected, but an adversary being
    able to access those two blocks (i.e. 64KB of the stored data)
    is bad enough. See Section 8.3 in
    https://nvlpubs.nist.gov/nistpubs/Legacy/SP/nistspecialpublication800-38d.pdf
    XChaCha20-Poly1305 does not suffer from this constraint and
    stays secure even if the filesystem gets very large.
  * New platforms:
  - CryFS now works on devices with Apple M1 silicon
  * Build changes:
  - Switch to Conan package manager
  - Allow an easy way to modify how the dependencies are found.
    This is mostly helpful for package maintainers. See "Using
    local dependencies" in the README.
  - Build with macFUSE instead of osxfuse on OSX
  - Now requires CMake 3.10 or later, and GCC 7 or later,
    or Clang 7 or later
  - Fix a build issue on Gentoo systems
  - Fix a build issue when building with boost 1.77
  * Improvements:
  - Display the file system configuration when mounting a file system
  - Now shows a better error message when failing to load the config
    file that distinguishes between "wrong password" and "config file
    not found".
  * New features:
  - Add support for atime mount options (noatime, strictatime,
    relatime, atime, nodiratime).
  - The new default is now noatime (in 0.10.x is was relatime).
    Noatime reduces the amount of writes necessary and with that
    reduces the probability of synchronization conflicts, and the
    probability of corrupted file systems if a power outage
    happens while writing.
  - Add an --immediate flag to cryfs-unmount that tries to unmount
    immediately and doesn't wait for processes to release their
    locks on the file system.
  - Add a --create-missing-basedir and --create-missing-mountpoint
    flag to create the base directory and mount directory respectively,
    if they don't exist, skipping the confirmation prompt.
  * Other:
  - Updated to spdlog 1.8.5
  - Updated to ranges-v3 0.11.0
  - Updated to boost 1.75
  - Updated to crypto++ 8.5
  * Clean up spec file
* Tue May 12 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Enable '-msse4.1' only on x86 and x86_64
* Thu Oct 17 2019 Richard Brown <rbrown@suse.com>
- Remove obsolete Groups tag (fate#326485)
* Wed Aug 14 2019 Michel Normand <normand@linux.vnet.ibm.com>
- Add _constraints for PowerPC for 4G disk space to avoid
  "No space left on device" error
* Thu Aug  1 2019 Michel Normand <normand@linux.vnet.ibm.com>
- disable lto for ppc64 bypass boo#1142574
* Sun Jul  7 2019 Klaas Freitag <opensuse@freisturz.de>
- Update to upstream version 0.10.2
  * Fix occasional crash in mkdir() on Windows
  * Fix a race condition when a file descriptor is closed while there's
    read/write requests for that file being processed.
  * Better logging when local state can't be loaded
  * Other fixes from version 0.10.1
* Fri Jul  5 2019 Michel Normand <normand@linux.vnet.ibm.com>
-  Do not set DCMAKE_CXX_FLAGS="-msse4.1" for PowerPC
* Fri Mar 22 2019 Klaas Freitag <opensuse@freisturz.de>
- Update to upstream version 0.10.0, selected changes are:
  * Integrity checks ensure you notice when someone modifies your file system.
  * File system nodes (files, directories, symlinks) store a parent
    pointer to the directory that contains them. This information can
    be used in later versions to resolve some synchronization conflicts.
  * Allow mounting using system mount tool and /etc/fstab
  * Performance improvements
  * Use relatime instead of strictatime (further performance improvement)
  * cryfs-unmount tool to unmount filesystems-
* Sat Feb  2 2019 Klaas Freitag <opensuse@freisturz.de>
- Update to upstream version 0.9.10
  * Fixed occasional deadlock (#64)
  * Fix for reading empty files out of bounds
  * Fixed race condition (#224 and #243)
* Tue Feb 27 2018 jengelh@inai.de
- Trim cleaning of buildroot, this is already implicit.
- Update description.
* Fri Feb  9 2018 klaas.freitag@suse.com
- Some spec file tweaks, based on Malcoms test build of cryfs
* Tue Feb  6 2018 klaas.freitag@suse.com
- Use -c switch in %%setup to create the source dir before unpacking
  the original tarball.
* Tue Feb  6 2018 klaas.freitag@suse.com
- Update to upstream version 0.9.9
* Mon Feb  5 2018 klaas.freitag@suse.com
- add a minimum version required
* Mon Feb  5 2018 klaas.freitag@suse.com
- use unversioned boost library.
* Thu Feb  1 2018 klaas.freitag@suse.com
- use boost version 1_58
* Thu Feb  1 2018 klaas.freitag@suse.com
- Initial commit
